# -*- coding: utf-8 -*-
##############################################################################
#
#    wms module for OpenERP, WMS Extended for Open ERP
#    Copyright (C) 2010 SYLEAM (<http://syleam.fr/>)
#              Christophe Chauvet <christophe.chauvet@syleam.fr>
#
#    This file is a part of wms
#
#    wms is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    wms is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv
from osv import fields
import netsvc

class StockMoveRemark(osv.osv):
    _name = 'stock.move.remark'
    _description = 'Remark for the stock move'

    _columns = {
        'name': fields.char('Name', size=128, translate=True, required=True),
        'code': fields.char('Code', size=16),
    }

    _defaults = {
        'name': lambda *a: False,
        'code': lambda *a: False,
    }

StockMoveRemark()


class StockMove(osv.osv):
    _inherit = 'stock.move'

    _columns = {
        'comment': fields.char('Comment', size=128, help='Enter comment, only if not present in remark list'),
        'remark_id': fields.many2one('stock.move.remark', 'Remark', help='Enter the remark for this product'),
        'purchase_price': fields.float('Last purchase price', help='Store the last purchase price for this product'),
        'average_price': fields.float('Average price', help='Store the weighted average price unit for this product'),
        'invoice_price': fields.float('Invoice price', help='Store the invoice price, check the invoice UOM as well'),
        'invoice_uom_id': fields.many2one('product.uom', 'Invoice UOM', help='Store the invoice UOM'),
    }

    _defaults = {
        'comment': lambda *a: False,
        'remark_id': lambda *a: False,
        'purchase_price': lambda *a: 0.0,
        'average_price': lambda *a: 0.0,
        'invoice_price': lambda *a: 0.0,
    }

    def check_assign(self, cr, uid, ids, context=None):
        """
        The assignement must be keep the production_lot and tracking
        and purpose 
        """
        done = []
        count = 0
        pickings = {}
        for move in self.browse(cr, uid, ids):
            if move.product_id.type == 'consu':
                if move.state in ('confirmed', 'waiting'):
                    done.append(move.id)
                pickings[move.picking_id.id] = 1
                continue
            if move.state in ('confirmed', 'waiting'):
                prodlot = move.prodlot_id and move.prodlot_id.id or False
                tracking = move.tracking_id and move.tracking_id.id or False
                res = self.pool.get('stock.location')._product_reserve(cr, uid, [move.location_id.id], move.product_id.id, move.product_qty, 
                        {'uom': move.product_uom.id, 'prodlot': prodlot, 'tracking': tracking})
                if res:
                    #_product_available_test depends on the next status for correct functioning
                    #the test does not work correctly if the same product occurs multiple times
                    #in the same order. This is e.g. the case when using the button 'split in two' of
                    #the stock outgoing form
                    self.write(cr, uid, move.id, {'state':'assigned'})
                    done.append(move.id)
                    pickings[move.picking_id.id] = 1
                    r = res.pop(0)
                    cr.execute('UPDATE stock_move SET location_id=%s, product_qty=%s WHERE id=%s', (r[1], r[0], move.id))

                    while res:
                        r = res.pop(0)
                        move_id = self.copy(cr, uid, move.id, {'product_qty': r[0], 'location_id': r[1]})
                        done.append(move_id)
                        #cr.execute('insert into stock_move_history_ids values (%s,%s)', (move.id,move_id))
        if done:
            count += len(done)
            self.write(cr, uid, done, {'state': 'assigned'})

        if count:
            for pick_id in pickings:
                wf_service = netsvc.LocalService("workflow")
                wf_service.trg_write(uid, 'stock.picking', pick_id, cr)
        return count


StockMove()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
