# -*- coding: utf-8 -*-
##############################################################################
#
#    wms module for OpenERP, WMS Extended for Open ERP
#    Copyright (C) 2010 SYLEAM (<http://www.syleam.fr/>)
#              Christophe CHAUVET <christophe.chauvet@syleam.fr>
#
#    This file is a part of wms
#
#    wms is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    wms is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv
from osv import fields

class Product(osv.osv):
    _inherit = 'product.product'

    _columns = {
        'analysis_id': fields.many2one('stock.analysis', 'ABC Class'),
        'warehouse_ids': fields.many2many('stock.warehouse', 'product_product_warehouse_rel', 'product_id', 'warehouse_id', 'Warehouses',
            help='Select the warehouse where the product must be stored',),
    }

    _defaults = {
        'warehouse_ids': lambda obj, cr, uid, context: obj.pool.get('res.users')._default_warehouse(cr, uid, {'M2M': True}),
    }

Product()

class ProductUL(osv.osv):
    _inherit = 'product.ul'

    _columns = {
        'length': fields.float('Length', help='Enter the length in cm'),
        'width': fields.float('Width', help='Enter the width en cm'),
        'height': fields.float('Height', help='Enter the height in cm'),
        'weight': fields.float('Weight', help='Enter the weight in Kg'),
        'volume': fields.float('Volume', help='Enter the volume in m3'),
        'product_id': fields.many2one('product.product', 'Product', help='Product for replenishment'),
        'is_inside': fields.boolean('Is content inside ?', help='Check this if the content is include in this volume'),
        'code': fields.char('Code', size=16, ),
        'warehouse_ids': fields.many2many('stock.warehouse', 'product_ul_warehouse_rel', 'ul_id', 'warehouse_id', 'Warehouses',
            help='Select the warehouse where the UL must be seen',),
    }

    _defaults = {
        'length': lambda *a: 0.0,
        'width': lambda *a: 0.0,
        'height': lambda *a: 0.0,
        'weight': lambda *a: 0.0,
        'volume': lambda *a: 0.0,
        'code': lambda *a: False,
        'warehouse_ids': lambda obj, cr, uid, context: obj.pool.get('res.users')._default_warehouse(cr, uid, {'M2M': True}),
    }

ProductUL()

class ProductPackaging(osv.osv):
    """
    We must redifine the name_get and name_search
    """
    _inherit = 'product.packaging'

    def name_get(self, cr, uid, ids, context=None):
        """Return EAN or Description or UL on many2one display"""
        if context is None:
            context = {}
        res = []
        for p in self.browse(cr, uid, ids, context=context):
            tmp = ''
            if p.ean:
                tmp += '[%s] ' % p.ean
            if p.name:
                tmp += p.name
            if not tmp:
                tmp = p.ul.name
            res.append((p.id, tmp))
        return res

    def name_search(self, cr, uid, name, args=None, operator='ilike', context=None, limit=80):
        """
        We must search product
        """
        if not args:
            args=[]
        if context is None:
            context={}
        ids = []
        if name:
            ids = self.search(cr, uid, [('ean','=',name)]+ args, limit=limit)
            if not ids:
                ids = self.search(cr, uid, [('ean','=',name)], limit=limit)
            if not ids:
                ids = self.search(cr, uid, [('name',operator,name)]+ args, limit=limit)
            if not ids:
                ids = self.search(cr, uid, [('name',operator,name)], limit=limit)
        else:
            ids = self.search(cr, uid, [('name',operator,name)]+ args, limit=limit)
            if not ids:
                ids = self.search(cr, uid, [('name',operator,name)], limit=limit)
        return self.name_get(cr, uid, ids, context=context)

    # TODO: when select a packaging with SSCC code, we return the product and this unit

ProductPackaging()


class ProductCategory(osv.osv):
    """
    Limit category display per warehouse on product
    """
    _inherit = 'product.category'

    _columns = {
        'warehouse_ids': fields.many2many('stock.warehouse', 'product_category_warehouse_rel',
            'categ_id', 'warehouse_id', 'Warehouses',
            help='Select the warehouse where the category must be seen',),
    }

    _defaults = {
        'warehouse_ids': lambda obj, cr, uid, context: obj.pool.get('res.users')._default_warehouse(cr, uid, {'M2M': True}),
    }

ProductCategory()


class ProductUOM(osv.osv):
    _inherit = 'product.uom'

    _columns = {
        'code': fields.char('Code', size=16, help='This field can be used on printing'),
        'warehouse_ids': fields.many2many('stock.warehouse', 'product_uom_warehouse_rel',
            'uom_id', 'warehouse_id', 'Warehouses',
            help='Select the warehouse where the category must be seen'),
    }

    _defaults = {
        'warehouse_ids': lambda obj, cr, uid, context: obj.pool.get('res.users')._default_warehouse(cr, uid, {'M2M': True}),
    }

ProductUOM()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
