# -*- coding: utf-8 -*-
##############################################################################
#
#    wms_inventory module for OpenERP, Module to extended inventory process
#    Copyright (C) 2010 SYLEAM (<http://www.syleam.fr/>)
#              Christophe CHAUVET <christophe.chauvet@syleam.fr>
#
#    This file is a part of wms_inventory
#
#    wms_inventory is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    wms_inventory is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv
from osv import fields
from osv.orm import except_orm
from tools.translate import _
import netsvc
import thread
import pooler
import time


class Inventory(osv.osv):
    _inherit = 'stock.inventory'

    _columns = {
        'type': fields.selection([('global','Global'),('partial','Partial')], 'Type', 
            help='If global, reset all quantity product  not found in this inventory'),
        'warehouse_id': fields.many2one('stock.warehouse', 'Warehouse', required=True,
            help='Select the warehouse where the inventory is made'),
    }

    _defaults = {
        'type': lambda *a: 'partial',
        'warehouse_id': lambda obj, cr, uid, context: obj.pool.get('res.users')._default_warehouse(cr, uid, context),
    }


    def action_done(self, cr, uid, ids, context=None):
        """
        Validate the inventory, it take a lot of time and must be launch in multi thread
        """
        def threading_done(cr, uid, ids, context=None):
            """
            Treatment launch as a new thread
            Must be include production lot and tracking
            """
            logger = netsvc.Logger()
            logger.notifyChannel('wms_inventory', netsvc.LOG_INFO, '[%s] Start compute the inventory *********' % cr.dbname) 
            cr = pooler.get_db(cr.dbname).cursor()
            for inv in self.browse(cr, uid, ids):
                logger.notifyChannel('wms_inventory', netsvc.LOG_INFO, '[%s] Inventory %s' % (cr.dbname, inv.name)) 
                move_ids = []
                move_line = []
                inv_ok = True
                inv_message = ''
                for line in inv.inventory_line_id:
                    pid = line.product_id.id
                    price = line.product_id.standard_price or 0.0
                    prodlot = line.prodlot_id and line.prodlot_id.id or False
                    tracking = line.tracking_id and line.tracking_id.id or False
                    amount = self.pool.get('stock.location')._stock_get(cr, uid, line.location_id.id, [pid], 
                            {'uom': line.product_uom.id, 'prodlot': prodlot, 'tracking': tracking})
                    change = line.product_qty - amount
                    if change:
                        location_id = line.product_id.product_tmpl_id.property_stock_inventory.id
                        value = {
                            'name': 'INV:' + str(line.inventory_id.id) + ':' + line.inventory_id.name,
                            'product_id': line.product_id.id,
                            'product_uom': line.product_uom.id,
                            'date': inv.date,
                            'date_planned': inv.date,
                            'state': 'assigned',
                            'prodlot_id': prodlot,
                            'tracking_id': tracking,
                        }
                        if change > 0:
                            value.update( {
                                'product_qty': change,
                                'location_id': location_id,
                                'location_dest_id': line.location_id.id,
                            })
                        else:
                            value.update( {
                                'product_qty': -change,
                                'location_id': line.location_id.id,
                                'location_dest_id': location_id,
                            })
                        try:
                            move_ids.append(self.pool.get('stock.move').create(cr, uid, value))
                        except except_orm, e:
                            logger.notifyChannel('wms_inventory', netsvc.LOG_INFO, '[%s] %s %s' % (cr.dbname, e.name, e.value)) 
                            inv_message += '    %s: %s\n' % (e.name, e.value)
                            inv_ok = False
                logger.notifyChannel('wms_inventory', netsvc.LOG_INFO, '[%s] Compute each line' % cr.dbname) 
                if len(move_ids) and inv_ok:
                    try:
                        self.pool.get('stock.move').action_done(cr, uid, move_ids,
                                context=context)
                    except except_orm, e:
                        logger.notifyChannel('wms_inventory', netsvc.LOG_INFO, '[%s] %s %s' % (cr.dbname, e.name, e.value)) 
                        inv_message += '    %s: %s\n' % (e.name, e.value)
                        inv_ok = False

                if inv_ok:
                    self.write(cr, uid, [inv.id], {'state': 'done', 'date_done': time.strftime('%Y-%m-%d %H:%M:%S'), 
                        'move_ids': [(6, 0, move_ids)]})

                ###
                ## Send a request to the user whom launch the validation
                if inv_ok:
                    body = _('Inventory %s has been finished') % inv.name
                else:
                    cr.rollback()
                    body = _('Inventory %s failed\nsee below\n') % inv.name
                    body += inv_message

                self.pool.get('res.request').create(cr, uid, {
                    'name': _('Compute inventory %s done') % inv.name,
                    'act_from': uid,
                    'act_to': uid,
                    'body' : body,
                }, context=context)

                cr.commit()
            cr.close()
            logger.notifyChannel('wms_inventory', netsvc.LOG_INFO, '[%s] End compute the inventory ***********' % cr.dbname) 
            return True

        thread.start_new_thread(threading_done, (cr, uid, ids, context))
        return True

Inventory()

class InventoryLine(osv.osv):
    _inherit = 'stock.inventory.line'

    _columns = {
        'prodlot_id': fields.many2one('stock.production.lot', 'Production Lot'),
        'tracking_id': fields.many2one('stock.tracking', 'Tracking'),
    }

    def on_change_product_id(self, cr, uid, ids, location_id, product, uom=False):
        if not product:
            return {}
        if not uom:
            prod = self.pool.get('product.product').browse(cr, uid, [product], {'uom': uom})[0]
            uom = prod.uom_id.id
        #amount = self.pool.get('stock.location')._product_get(cr, uid, location_id, [product], {'uom': uom})[product]
        result = {'product_uom': uom}
        return {'value': result}

InventoryLine()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
