# -*- coding: utf-8 -*-
##############################################################################
#
#    wms_location module for OpenERP, Location management
#    Copyright (C) 2010 SYLEAM (<http://syleam.fr/>)
#              Christophe Chauvet <christophe.chauvet@syleam.fr>
#
#    This file is a part of wms_location
#
#    wms_location is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    wms_location is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv
from osv import fields

class StockLocation(osv.osv):
    """
    Add new fields on stock location
    """
    _inherit = 'stock.location'

    _columns = {
        'code': fields.char('Code', size=32, help='Enter the complete code for this location, usefull for search'),
        'warehouse_id': fields.many2one('stock.warehouse', 'Warehouse'),
        'width': fields.float('Width', help='Enter the width in cm'),
        'height': fields.float('Height', help='Enter the height in cm'),
        'depth': fields.float('depth', help='Enter the depth in cm'),
        'transcend_width': fields.boolean('Transcend Width', help='Autorize the width to be transcend'),
        'transcend_height': fields.boolean('Transcend Height', help='Autorize the height to be transcend'),
        'transcend_depth': fields.boolean('Transcend Depth', help='Autorize the depth to be transcend'),
    }

    _defaults = {
        'code': lambda *a: False,
        'width': lambda *a: 0.0,
        'height': lambda *a: 0.0,
        'transcend_width': lambda *a: False,
        'transcend_height': lambda *a: False,
        'transcend_depth': lambda *a: False,
    }

    def _stock_get(self, cr, uid, location_id, product_id, context=None):
        """
        We must retrieve the stop at this location and for this production_lot or tracking ID
        """
        # TODO: Convert unit if different
        move_obj = self.pool.get('stock.move')
        args = [
            ('location_dest_id', '=', location_id),
            ('product_id', '=', product_id[0]),
            ('product_uom', '=', context.get('uom')),
            ('prodlot_id','=', context.get('prodlot', False)),
            ('tracking_id','=', context.get('tracking', False)),
        ]
        move_ids = move_obj.search(cr, uid, args, context=context)
        move = move_obj.read(cr, uid, move_ids, ['product_qty'], context=context)
        res = 0.0
        for m in move:
            res += m['product_qty']
        return res


    def _product_reserve(self, cr, uid, ids, product_id, product_qty, context={}):
        """
        We must really compute stock with production lot and tracking
        tracking have the highest priority per production lot
        """
        result = []
        amount = 0.0
        loc_ids = self.search(cr, uid, [('location_id', 'child_of', ids)])

        arg_query1 = [tuple(loc_ids)]
        sql_query1 = """SELECT coalesce(sum(m11.product_qty), 0)
            FROM   stock_move m11
            WHERE  m11.state = m0.state
            AND    m11.product_id = m0.product_id
            AND    m11.location_dest_id = m0.location_dest_id
            AND    m11.location_id NOT IN %s"""

        arg_query2 = [tuple(loc_ids)]
        sql_query2 = """SELECT coalesce(sum(m13.product_qty),0)
        FROM   stock_move m13
        WHERE  m13.state = m0.state
        AND    m13.product_id = m0.product_id
        AND    m13.location_id IN %s
        AND    m13.location_dest_id = m0.location_dest_id"""

        arg_query3 = [tuple(loc_ids)]
        sql_query3 = """SELECT coalesce(sum(m12.product_qty), 0)
        FROM   stock_move m12
        WHERE  m12.state = m0.state
        AND    m12.product_id = m0.product_id
        AND    m12.location_id = m0.location_dest_id
        AND    m12.location_dest_id IN %s"""

        arg_query4 = [tuple(loc_ids)]
        sql_query4 = """SELECT coalesce(sum(m14.product_qty), 0)
        FROM   stock_move m14
        WHERE  m14.state= m0.state
        AND    m14.product_id = m0.product_id
        AND    m14.location_id = m0.location_dest_id
        AND    m14.location_dest_id not in %s"""

        if context.get('prodlot'):
            arg_query1.append(context['prodlot'])
            sql_query1 += '\nAND prodlot_id=%s'
            arg_query2.append(context['prodlot'])
            sql_query2 += '\nAND prodlot_id=%s'
            arg_query3.append(context['prodlot'])
            sql_query3 += '\nAND prodlot_id=%s'
            arg_query4.append(context['prodlot'])
            sql_query4 += '\nAND prodlot_id=%s'

        if context.get('tracking'):
            arg_query1.append(context['tracking'])
            sql_query1 += '\nAND tracking_id=%s'
            arg_query2.append(context['tracking'])
            sql_query2 += '\nAND tracking_id=%s'
            arg_query3.append(context['tracking'])
            sql_query3 += '\nAND tracking_id=%s'
            arg_query4.append(context['tracking'])
            sql_query4 += '\nAND tracking_id=%s'

        sql_query = """SELECT * FROM (
        SELECT m0.location_dest_id, m0.product_uom,
               (%s) + (%s) as in_qty,
               (%s) + (%s) as out_qty """ % (sql_query1, sql_query2, sql_query3, sql_query4)

        arg_query = [product_id, tuple(loc_ids)]
        sql_query += """FROM   stock_move m0
        WHERE  m0.state='done'
        AND    m0.product_id=%s
        AND    location_dest_id IN %s"""

        if context.get('prodlot'):
            arg_query.append(context['prodlot'])
            sql_query += '\nAND prodlot_id=%s'

        if context.get('tracking'):
            arg_query.append(context['tracking'])
            sql_query += '\nAND tracking_id=%s'

        sql_query += """\nGROUP by
               location_dest_id,
               m0.state,
               m0.product_id,"""

        if context.get('prodlot'):
            sql_query += '\nm0.prodlot_id,'

        if context.get('tracking'):
            sql_query += '\nm0.tracking_id,'

        sql_query += """\nm0.product_uom
        ) s
        WHERE s.in_qty > s.out_qty"""

        cr.execute(sql_query, tuple(arg_query1 + arg_query2 + arg_query3 + arg_query4 + arg_query))
        results = cr.dictfetchall()
        if not results:
            return False

        total = 0.0
        for r in results:
            amount = self.pool.get('product.uom')._compute_qty(cr, uid, r['product_uom'], r['in_qty'] - r['out_qty'], context.get('uom', False))
            total += amount
            if total >= product_qty:
                amount = amount - (total - product_qty)
                result.append((amount, r['location_dest_id']))
                break
            result.append((amount, r['location_dest_id']))
        return result

StockLocation()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
