# -*- coding: utf-8 -*-
##############################################################################
#
#    wms_scanner module for OpenERP, Module for manage barcode reader
#    Copyright (C) 2010 SYLEAM (<http://syleam.fr/>)
#              Christophe Chauvet <christophe.chauvet@syleam.fr>
#
#    This file is a part of wms_scanner
#
#    wms_scanner is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    wms_scanner is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv
from osv import fields

class ScannerHardware(osv.osv):
    _name = 'scanner.hardware'
    _description = 'Scanner Hardware'

    _columns = {
        'name': fields.char('Name', size=128, required=True),
        'code': fields.char('Code', size=12, required=True),
        'warehouse_id': fields.many2one('stock.warehouse', 'Warehouse', required=True),
        'scenario_id': fields.many2one('scanner.scenario', 'Scenario'),
    }

    def scanner_call(self, cr, uid, numterm=None, action=None, message=''):
        """
        This method is the only one, that called by the barcode reader,

        """
        if numterm is None:
            raise osv.except_osv('Error', 'Action must be required')

        if action is None:
            raise osv.except_osv('Error', 'Action must be required')

        if action == 'menu':
            return ('M', ['line1','line2','line3'])
        elif action == 'message':
            # Retrieve scanning for analyse
            return ('Q', {'label': ['Can you confirm', '_the quantity'], 'value': (3, 16)})

ScannerHardware()

class ScannerScenario(osv.osv):
    _name = 'scanner.scenario'
    _description = 'Scenario for scanner'

    _columns = {
        'name': fields.char('Name', size=64, ),
        'sequence': fields.integer('Sequence'),
        'active': fields.boolean('Active', help='if check, this object is always available'),
        'model_id': fields.many2one('ir.model', 'Model', required=True),
        'step_ids': fields.one2many('scanner.scenario.step', 'scenario_id', 'Scenario', ),
        'warehouse_ids': fields.many2many('stock.warehouse', 'scanner_scenario_warehouse_rel', 'scenario_id', 'warehouse_id', 'Warehouses'),
    }

ScannerScenario()


class ScannerScenarioStep(osv.osv):
    _name = 'scanner.scenario.step'
    _description = 'Step for scenario'

    _columns = {
        'name': fields.char('Name', size=64, help='Name of the step'),
        'scenario_id': fields.many2one('scanner.scenario', 'Scenario', required=True),
        'step_start': fields.boolean('Step start',help='Check this if this is the first step of scenario'),
        'step_stop': fields.boolean('Step stop',help='check this if this is the last step of scenario'),
        'out_trans_ids': fields.one2many('scanner.scenario.transition', 'from_id', 'Outgoing transitions'),
        'in_trans_ids': fields.one2many('scanner.scenario.transition', 'to_id', 'Incoming transitions'),
    }

    _defaults = {
        'step_start': lambda *a: False,
        'step_stop': lambda *a: False,
    }

ScannerScenarioStep()


class ScannerScenarioTransition(osv.osv):
    _name = 'scanner.scenario.transition'
    _description = 'Transition for scenario'

    _columns = {
        'name': fields.char('Name', size=64, required=True),
        'from_id': fields.many2one('scanner.scenario.step', 'From', required=True),
        'to_id': fields.many2one('scanner.scenario.step', 'To', required=True),
        'condition': fields.char('Condition', size=256, required=True),
    }

    _defaults = {
        'condition': lambda *a: 'True',
    }

ScannerScenarioTransition()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
